@@ -22,8 +22,8 @@ from pydantic import BaseModel
 from railtracks.built_nodes.concrete import (
     DynamicFunctionNode,
     LLMBase,
-    OutputLessToolCallLLM,
 )
+from railtracks.built_nodes.concrete._tool_call_base import OutputLessToolCallLLMBase
 from railtracks.llm import (
     ModelBase,
     Parameter,
@@ -32,7 +32,6 @@ from railtracks.llm import (
 )
 from railtracks.llm.type_mapping import TypeMapper
 from railtracks.nodes.nodes import Node
-from railtracks.utils.visuals.browser.chat_ui import ChatUI
 from railtracks.validation.node_creation.validation import (
     _check_duplicate_param_names,
     _check_max_tool_calls,
@@ -143,7 +142,7 @@ class NodeBuilder(Generic[_TNode]):
         Raises:
             AssertionError: If the node class is not a subclass of a ToolCallingLLM in the RT framework.
         """
-        assert issubclass(self._node_class, OutputLessToolCallLLM), (
+        assert issubclass(self._node_class, OutputLessToolCallLLMBase), (
             f"To perform this operation the node class we are building must be of type LLMBase but got {self._node_class}"
         )
 
@@ -165,22 +164,6 @@ class NodeBuilder(Generic[_TNode]):
         self._with_override("tool_nodes", classmethod(lambda cls: connected_nodes))
         self._with_override("max_tool_calls", max_tool_calls)
 
-    def chat_ui(
-        self,
-        chat_ui: ChatUI,
-    ):
-        """
-        Configure a chat UI for the node.
-
-        Starts the chat UI server asynchronously and sets it as an override
-        for the node being built.
-
-        Args:
-            chat_ui (ChatUI): The chat UI instance to configure for this node.
-        """
-        chat_ui.start_server_async()
-        self._with_override("chat_ui", chat_ui)
-
     @overload
     def setup_function_node(
         self,
@@ -331,11 +314,9 @@ class NodeBuilder(Generic[_TNode]):
             f"You tried to add prepare_tool_llm to a non LLM Node of {type(self._node_class)}."
         )
 
-        def prepare_tool(cls, tool_parameters: Dict[str, Any]):
+        def prepare_tool(cls, **kwargs):
             # Use the shared implementation in LLMBase
-            message_hist = cls.prepare_tool_message_history(
-                tool_parameters, tool_params
-            )
+            message_hist = cls.prepare_tool_message_history(kwargs, tool_params)
             return cls(message_hist)
 
         self._with_override("prepare_tool", classmethod(prepare_tool))
