@@ -1,9 +1,11 @@
 from ._call import call
 from .batch import call_batch
 from .broadcast_ import broadcast
+from .interactive import local_chat
 
 __all__ = [
     "call",
     "call_batch",
     "broadcast",
+    "local_chat",
 ]
