@@ -9,9 +9,12 @@ from .models import (
     GeminiLLM,
     HuggingFaceLLM,
     OllamaLLM,
+    OpenAICompatibleProvider,
     OpenAILLM,
+    PortKeyLLM,
+    # TelusLLM,
 )
-from .models.providers import ModelProvider
+from .providers import ModelProvider
 from .tools import (
     ArrayParameter,
     ObjectParameter,
@@ -42,6 +45,10 @@ __all__ = [
     "OllamaLLM",
     "AzureAILLM",
     "GeminiLLM",
+    # "TelusLLM",
+    "PortKeyLLM",
+    "OpenAICompatibleProvider",
+    "CohereLLM",
     # Parameter types
     "Parameter",
     "UnionParameter",
