@@ -2,17 +2,32 @@
 # In the following document, we will use the interface types defined in this module to interact with the llama index to
 # route to a given model.
 ###
+from __future__ import annotations
+
 from abc import ABC, abstractmethod
-from typing import Callable, List, Type
+from typing import (
+    AsyncGenerator,
+    Callable,
+    Generator,
+    Generic,
+    List,
+    Literal,
+    Type,
+    TypeVar,
+    overload,
+)
 
 from pydantic import BaseModel
 
 from .history import MessageHistory
+from .providers import ModelProvider
 from .response import Response
 from .tools import Tool
 
+_TStream = TypeVar("_TStream", Literal[True], Literal[False])
 
-class ModelBase(ABC):
+
+class ModelBase(ABC, Generic[_TStream]):
     """
     A simple base that represents the behavior of a model that can be used for chat, structured interactions, and streaming.
 
@@ -24,23 +39,32 @@ class ModelBase(ABC):
 
     def __init__(
         self,
-        pre_hooks: List[Callable[[MessageHistory], MessageHistory]] | None = None,
-        post_hooks: List[Callable[[MessageHistory, Response], Response]] | None = None,
-        exception_hooks: List[Callable[[MessageHistory, Exception], None]]
+        __pre_hooks: List[Callable[[MessageHistory], MessageHistory]] | None = None,
+        __post_hooks: List[Callable[[MessageHistory, Response], Response]]
         | None = None,
+        __exception_hooks: List[Callable[[MessageHistory, Exception], None]]
+        | None = None,
+        stream: _TStream = False,
     ):
-        if pre_hooks is None:
+        if __pre_hooks is None:
             pre_hooks: List[Callable[[MessageHistory], MessageHistory]] = []
+        else:
+            pre_hooks = __pre_hooks
 
-        if post_hooks is None:
+        if __post_hooks is None:
             post_hooks: List[Callable[[MessageHistory, Response], Response]] = []
+        else:
+            post_hooks = __post_hooks
 
-        if exception_hooks is None:
+        if __exception_hooks is None:
             exception_hooks: List[Callable[[MessageHistory, Exception], None]] = []
+        else:
+            exception_hooks = __exception_hooks
 
         self._pre_hooks = pre_hooks
         self._post_hooks = post_hooks
         self._exception_hooks = exception_hooks
+        self.stream = stream
 
     def add_pre_hook(self, hook: Callable[[MessageHistory], MessageHistory]) -> None:
         """Adds a pre-hook to modify messages before sending them to the model."""
@@ -79,10 +103,19 @@ class ModelBase(ABC):
         """
         pass
 
+    @abstractmethod
+    def model_provider(self) -> ModelProvider:
+        """The name of the provider of this model (The Company that owns the model)"""
+        pass
+
     @classmethod
     @abstractmethod
-    def model_type(cls) -> str:
-        """The name of the provider of this model or the model type."""
+    def model_gateway(cls) -> ModelProvider:
+        """
+        Gets the API distrubutor of the model. Note nessecarily the same as the model itself.
+
+        E.g. if you are calling openai LLM through Azure AI foundry
+        """
         pass
 
     def _run_pre_hooks(self, message_history: MessageHistory) -> MessageHistory:
@@ -106,157 +139,241 @@ class ModelBase(ABC):
         for hook in self._exception_hooks:
             hook(message_history, exception)
 
-    def chat(self, messages: MessageHistory, **kwargs):
+    def generator_wrapper(
+        self,
+        generator: Generator[str | Response, None, Response],
+        message_history: MessageHistory,
+    ) -> Generator[str | Response, None, Response]:
+        new_response: Response | None = None
+        for g in generator:
+            if isinstance(g, Response):
+                g.message_info
+                new_response = self._run_post_hooks(message_history, g)
+                yield new_response
+
+            yield g
+
+        assert new_response is not None, (
+            "The generator did not yield a final Response object so nothing could be done."
+        )
+
+        return new_response
+
+    @overload
+    def chat(self: ModelBase[Literal[False]], messages: MessageHistory) -> Response:
+        pass
+
+    @overload
+    def chat(
+        self: ModelBase[Literal[True]], messages: MessageHistory
+    ) -> Generator[str | Response, None, Response]:
+        pass
+
+    def chat(
+        self, messages: MessageHistory
+    ) -> Response | Generator[str | Response, None, Response]:
         """Chat with the model using the provided messages."""
 
         messages = self._run_pre_hooks(messages)
 
         try:
-            response = self._chat(messages, **kwargs)
+            response = self._chat(messages)
         except Exception as e:
             self._run_exception_hooks(messages, e)
             raise e
 
+        if isinstance(response, Generator):
+            return self.generator_wrapper(response, messages)
+
         response = self._run_post_hooks(messages, response)
         return response
 
-    async def achat(self, messages: MessageHistory, **kwargs):
+    @overload
+    async def achat(
+        self: ModelBase[Literal[False]], messages: MessageHistory
+    ) -> Response:
+        pass
+
+    @overload
+    async def achat(
+        self: ModelBase[Literal[True]], messages: MessageHistory
+    ) -> Generator[str | Response, None, Response]:
+        pass
+
+    async def achat(self, messages: MessageHistory):
         """Asynchronous chat with the model using the provided messages."""
         messages = self._run_pre_hooks(messages)
 
         try:
-            response = await self._achat(messages, **kwargs)
+            response = await self._achat(messages)
         except Exception as e:
             self._run_exception_hooks(messages, e)
             raise e
 
+        if isinstance(response, Generator):
+            return self.generator_wrapper(response, messages)
+
         response = self._run_post_hooks(messages, response)
 
         return response
 
-    def structured(self, messages: MessageHistory, schema: Type[BaseModel], **kwargs):
+    @overload
+    def structured(
+        self: ModelBase[Literal[False]],
+        messages: MessageHistory,
+        schema: Type[BaseModel],
+    ) -> Response:
+        pass
+
+    @overload
+    def structured(
+        self: ModelBase[Literal[True]],
+        messages: MessageHistory,
+        schema: Type[BaseModel],
+    ) -> Generator[str | Response, None, Response]:
+        pass
+
+    def structured(self, messages: MessageHistory, schema: Type[BaseModel]):
         """Structured interaction with the model using the provided messages and output_schema."""
         messages = self._run_pre_hooks(messages)
 
         try:
-            response = self._structured(messages, schema, **kwargs)
+            response = self._structured(messages, schema)
         except Exception as e:
             self._run_exception_hooks(messages, e)
             raise e
 
+        if isinstance(response, Generator):
+            return self.generator_wrapper(response, messages)
+
         response = self._run_post_hooks(messages, response)
 
         return response
 
+    @overload
     async def astructured(
-        self, messages: MessageHistory, schema: Type[BaseModel], **kwargs
-    ):
+        self: ModelBase[Literal[False]],
+        messages: MessageHistory,
+        schema: Type[BaseModel],
+    ) -> Response:
+        pass
+
+    @overload
+    async def astructured(
+        self: ModelBase[Literal[True]],
+        messages: MessageHistory,
+        schema: Type[BaseModel],
+    ) -> Generator[str | Response, None, Response]:
+        pass
+
+    async def astructured(self, messages: MessageHistory, schema: Type[BaseModel]):
         """Asynchronous structured interaction with the model using the provided messages and output_schema."""
         messages = self._run_pre_hooks(messages)
 
         try:
-            response = await self._astructured(messages, schema, **kwargs)
+            response = await self._astructured(messages, schema)
         except Exception as e:
             self._run_exception_hooks(messages, e)
             raise e
 
+        if isinstance(response, Generator):
+            return self.generator_wrapper(response, messages)
+
         response = self._run_post_hooks(messages, response)
 
         return response
 
-    def stream_chat(self, messages: MessageHistory, **kwargs):
-        """Stream chat with the model using the provided messages."""
-        messages = self._run_pre_hooks(messages)
+    @overload
+    def chat_with_tools(
+        self: ModelBase[Literal[False]], messages: MessageHistory, tools: List[Tool]
+    ) -> Response:
+        pass
 
-        try:
-            response = self._stream_chat(messages, **kwargs)
-        except Exception as e:
-            self._run_exception_hooks(messages, e)
-            raise e
+    @overload
+    def chat_with_tools(
+        self: ModelBase[Literal[True]], messages: MessageHistory, tools: List[Tool]
+    ) -> Generator[str | Response, None, Response]:
+        pass
 
-        response = self._run_post_hooks(messages, response)
-
-        return response
-
-    async def astream_chat(self, messages: MessageHistory, **kwargs):
-        """Asynchronous stream chat with the model using the provided messages."""
-        messages = self._run_pre_hooks(messages)
-
-        try:
-            response = await self._astream_chat(messages, **kwargs)
-        except Exception as e:
-            self._run_exception_hooks(messages, e)
-            raise e
-
-        response = self._run_post_hooks(messages, response)
-
-        return response
-
-    def chat_with_tools(self, messages: MessageHistory, tools: List[Tool], **kwargs):
+    def chat_with_tools(self, messages: MessageHistory, tools: List[Tool]):
         """Chat with the model using the provided messages and tools."""
         messages = self._run_pre_hooks(messages)
 
         try:
-            response = self._chat_with_tools(messages, tools, **kwargs)
+            response = self._chat_with_tools(messages, tools)
         except Exception as e:
             self._run_exception_hooks(messages, e)
             raise e
 
+        if isinstance(response, Generator):
+            return self.generator_wrapper(response, messages)
+
         response = self._run_post_hooks(messages, response)
         return response
 
+    @overload
     async def achat_with_tools(
-        self, messages: MessageHistory, tools: List[Tool], **kwargs
-    ):
+        self: ModelBase[Literal[False]], messages: MessageHistory, tools: List[Tool]
+    ) -> Response:
+        pass
+
+    @overload
+    async def achat_with_tools(
+        self: ModelBase[Literal[True]], messages: MessageHistory, tools: List[Tool]
+    ) -> Generator[str | Response, None, Response]:
+        pass
+
+    async def achat_with_tools(self, messages: MessageHistory, tools: List[Tool]):
         """Asynchronous chat with the model using the provided messages and tools."""
         messages = self._run_pre_hooks(messages)
 
         try:
-            response = await self._achat_with_tools(messages, tools, **kwargs)
+            response = await self._achat_with_tools(messages, tools)
         except Exception as e:
             self._run_exception_hooks(messages, e)
             raise e
 
+        if isinstance(response, Generator):
+            return self.generator_wrapper(response, messages)
+
         response = self._run_post_hooks(messages, response)
 
         return response
 
     @abstractmethod
-    def _chat(self, messages: MessageHistory, **kwargs) -> Response:
+    def _chat(
+        self, messages: MessageHistory
+    ) -> Response | Generator[str | Response, None, Response]:
         pass
 
     @abstractmethod
     def _structured(
-        self, messages: MessageHistory, schema: Type[BaseModel], **kwargs
-    ) -> Response:
-        pass
-
-    @abstractmethod
-    def _stream_chat(self, messages: MessageHistory, **kwargs) -> Response:
+        self, messages: MessageHistory, schema: Type[BaseModel]
+    ) -> Response | Generator[str | Response, None, Response]:
         pass
 
     @abstractmethod
     def _chat_with_tools(
-        self, messages: MessageHistory, tools: List[Tool], **kwargs
-    ) -> Response:
+        self, messages: MessageHistory, tools: List[Tool]
+    ) -> Response | Generator[str | Response, None, Response]:
         pass
 
     @abstractmethod
-    async def _achat(self, messages: MessageHistory, **kwargs) -> Response:
+    async def _achat(
+        self, messages: MessageHistory
+    ) -> Response | AsyncGenerator[str | Response, None]:
         pass
 
     @abstractmethod
     async def _astructured(
-        self, messages: MessageHistory, schema: Type[BaseModel], **kwargs
-    ) -> Response:
-        pass
-
-    @abstractmethod
-    async def _astream_chat(self, messages: MessageHistory, **kwargs) -> Response:
+        self,
+        messages: MessageHistory,
+        schema: Type[BaseModel],
+    ) -> Response | AsyncGenerator[str | Response, None]:
         pass
 
     @abstractmethod
     async def _achat_with_tools(
-        self, messages: MessageHistory, tools: List[Tool], **kwargs
-    ) -> Response:
+        self, messages: MessageHistory, tools: List[Tool]
+    ) -> Response | AsyncGenerator[str | Response, None]:
         pass
