@@ -5,13 +5,22 @@ This package provides classes and utilities for creating tools from Python funct
 handling various parameter types, and parsing docstrings.
 """
 
-from .parameter import ArrayParameter, Parameter, ParameterType, PydanticParameter
+from .parameters import (
+    ArrayParameter,
+    ObjectParameter,
+    Parameter,
+    ParameterType,
+    RefParameter,
+    UnionParameter,
+)
 from .tool import Tool
 
 __all__ = [
     "Parameter",
-    "PydanticParameter",
     "ArrayParameter",
     "ParameterType",
+    "ObjectParameter",
+    "UnionParameter",
+    "RefParameter",
     "Tool",
 ]
