@@ -4,13 +4,14 @@ import pytest
 import railtracks as rt
 from railtracks.llm import Message, MessageHistory
 from railtracks.llm.response import Response
+from railtracks.llm.message import Role
 
 
 def test_prompt_injection(mock_llm):
     prompt = "{secret}"
 
     def return_message(messages: MessageHistory) -> Response:
-        return Response(message=Message(role="assistant", content=messages[-1].content))
+        return Response(message=Message(role=Role.assistant, content=messages[-1].content))
 
     model = mock_llm()
     model._chat = return_message
@@ -30,7 +31,7 @@ def test_prompt_injection_bypass(mock_llm):
     prompt = "{{secret_value}}"
 
     def return_message(messages: MessageHistory) -> Response:
-        return Response(message=Message(role="assistant", content=messages[-1].content))
+        return Response(message=Message(role=Role.assistant, content=messages[-1].content))
 
     model = mock_llm()
     model._chat = return_message
@@ -51,7 +52,7 @@ def test_prompt_numerical(mock_llm):
     prompt = "{1}"
 
     def return_message(messages: MessageHistory) -> Response:
-        return Response(message=Message(role="assistant", content=messages[-1].content))
+        return Response(message=Message(role=Role.assistant, content=messages[-1].content))
 
     model = mock_llm()
     model._chat = return_message
@@ -75,7 +76,7 @@ def test_prompt_not_in_context(mock_llm):
     prompt = "{secret2}"
 
     def return_message(messages: MessageHistory) -> Response:
-        return Response(message=Message(role="assistant", content=messages[-1].content))
+        return Response(message=Message(role=Role.assistant, content=messages[-1].content))
 
     model = mock_llm()
     model._chat = return_message
@@ -101,7 +102,7 @@ def test_prompt_injection_global_config_bypass(mock_llm):
     prompt = "{secret_value}"
 
     def return_message(messages: MessageHistory) -> Response:
-        return Response(message=Message(role="assistant", content=messages[-1].content))
+        return Response(message=Message(role=Role.assistant, content=messages[-1].content))
 
     model = mock_llm()
     model._chat = return_message
